// ephemera/colour.cpp
// Simple colour class.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#include "global.h"

#include "colour.h"


namespace eph
{


//
// Set the colour from HSV values.
//

void
rgb::from_hsv( double h, double s, double v )
{
	// Ensure all components are in [0, 1].
	h = clamp( h, 0.0, 1.0 );
	s = clamp( s, 0.0, 1.0 );
	v = clamp( v, 0.0, 1.0 );

	// Convert.
	int hi = clamp( (int) floor( h*6 ), 0, 5 );
	double f = h*6 - floor( h*6 );
	double p = v*(1-s);
	double q = v*(1-f*s);
	double t = v*(1-(1-f)*s);
	
	// Scale the components into the byte range.
	unsigned char v0 = (unsigned char) clamp( (int) floor( v*256 ), 0, 255 );
	unsigned char p0 = (unsigned char) clamp( (int) floor( p*256 ), 0, 255 );
	unsigned char q0 = (unsigned char) clamp( (int) floor( q*256 ), 0, 255 );
	unsigned char t0 = (unsigned char) clamp( (int) floor( t*256 ), 0, 255 );

	// Build the RGB colour.
	switch ( hi )
	{
	default:
		assert( !"Internal error in rgb::from_hsv calculation." );
		// Fall through.
	case 0:
		set( v0, t0, p0 );
		break;
		
	case 1:
		set( q0, v0, p0 );
		break;
		
	case 2:
		set( p0, v0, t0 );
		break;
		
	case 3:
		set( p0, q0, v0 );
		break;
		
	case 4:
		set( t0, p0, v0 );
		break;
		
	case 5:
		set( v0, p0, q0 );
		break;
	}
}


}; // namespace eph
