// ephemera/debug.h
// Override assert for screen savers.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#ifndef EPHEMERA_DEBUG_H
#define EPHEMERA_DEBUG_H


namespace eph
{


// Screen savers need special assert handling since they run full screen and
// topmost. This assert will beep the first time an assert triggers, and will
// log all failures to the debugger's output window.
#undef assert
#ifdef NDEBUG
	#define assert( C )
#else // NDEBUG
	extern void assert_report_failure( const char *condition, const char *file, int line );
	#define assert( C )		(void) ( (C) || (::eph::assert_report_failure( #C, __FILE__, __LINE__ ), 0) )
#endif // NDEBUG


}; // namespace eph


#endif // EPHEMERA_DEBUG_H
