// ephemera/ephemera.h
// Ephemera screen saver.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#ifndef EPHEMERA_EPHEMERA_H
#define EPHEMERA_EPHEMERA_H


#include "screen_saver.h"


namespace eph
{


// Forwards.
class animator;


// Ephemera screen saver.
class ephemera : public screen_saver
{
public:

	ephemera();
		// Construction.

	~ephemera();
		// Destruction.

	virtual int configure( HWND parent );
		// Display the configuration dialog.

	virtual bool saver_init( HWND hwnd, int width, int height );
		// Initialize the screen saver.

	virtual void saver_draw( HDC hdc, const RECT &dirty );
		// Draw the window while running as a screen saver.

	virtual bool saver_animate();
		// Perform animation processing.

private:

	static BOOL CALLBACK configure_dialog_proc( HWND hwnd, UINT msg, WPARAM wparam,  LPARAM lparam );
		// Dialog proc for the configuration dialog.

	int m_width;
	int m_height;
		// Dimensions of the screen saver window.

	HDC m_offscreen_dc;
	HBITMAP m_offscreen_bitmap;
	unsigned char *m_raw_bitmap;
		// Off-screen bitmap used to draw the fractals.

	animator *m_animator;
	DWORD m_last_animator_call;
		// Drawing management.
};


}; // namespace eph


#endif // EPHEMERA_EPHEMERA_H
