// ephemera/geometry.cpp
// Geometrical types: points and rectangles.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#include "global.h"

#include "geometry.h"


namespace eph
{


//
// Add pt to the area covered by the rectangle.
//

void
rectangle::include( const point &pt )
{
	if ( pt.x() < m_min_x )
	{
		m_min_x = pt.x();
	}
	else if ( pt.x() > m_max_x )
	{
		m_max_x = pt.x();
	}

	if ( pt.y() < m_min_y )
	{
		m_min_y = pt.y();
	}
	else if ( pt.y() > m_max_y )
	{
		m_max_y = pt.y();
	}
}


//
// Offset the position of the rectangle by pt.
//

void
rectangle::offset( const point &pt )
{
	m_min_x += pt.x();
	m_max_x += pt.x();
	m_min_y += pt.y();
	m_max_y += pt.y();
}


//
// Increase the size of the rectangle to match the given aspect ratio.
//

void
rectangle::inflate_to_aspect_ratio( const double &aspect_ratio )
{
	assert( epsilon_pos( width() ) );
	assert( epsilon_pos( height() ) );
	assert( epsilon_pos( aspect_ratio ) );

	if ( epsilon_pos( width() ) && epsilon_pos( height() ) )
	{
		// Inflate in X.
		double current_ratio = width() / height();
		if ( current_ratio < aspect_ratio )
		{
			double new_width = height() * aspect_ratio;
			assert( new_width > width() );
			double centre_x = min_x() + 0.5 * width();
			min_x( centre_x - 0.5 * new_width );
			max_x( centre_x + 0.5 * new_width );
		}
		// Inflate in Y.
		else
		{
			double new_height = width() / aspect_ratio;
			assert( new_height > height() );
			double centre_y = min_y() + 0.5 * height();
			min_y( centre_y - 0.5 * new_height );
			max_y( centre_y + 0.5 * new_height );
		}
	}
}


}; // namespace eph
