// ephemera/geometry.h
// Geometrical types: points and rectangles.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#ifndef EPHEMERA_GEOMETRY_H
#define EPHEMERA_GEOMETRY_H


namespace eph
{


// Simple point.
class point
{
public:

	point() {}
	point( const double &x, const double &y ) : m_x( x ), m_y( y ) {}
		// Construction.

	const double &x() const { return m_x; }
	void x( const double &new_x ) { m_x = new_x; }
	const double &y() const { return m_y; }
	void y( const double &new_y ) { m_y = new_y; }
		// Access to the components.

	void xy( const double &new_x, const double &new_y ) { m_x = new_x; m_y = new_y; }
		// Set both X and Y.

private:

	double m_x;
	double m_y;
		// X and Y components of the point.
};


//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


// Simple rectangle.
class rectangle
{
public:

	rectangle() {}
		// Construction.

	const double &min_x() const { return m_min_x; }
	void min_x( const double &new_x ) { m_min_x = new_x; }
	const double &min_y() const { return m_min_y; }
	void min_y( const double &new_y ) { m_min_y = new_y; }
	const double &max_x() const { return m_max_x; }
	void max_x( const double &new_x ) { m_max_x = new_x; }
	const double &max_y() const { return m_max_y; }
	void max_y( const double &new_y ) { m_max_y = new_y; }
	void min( const point &pt ) { m_min_x = pt.x(); m_min_y = pt.y(); }
	void max( const point &pt ) { m_max_x = pt.x(); m_max_y = pt.y(); }
		// Access to the components.

	double width() const { return m_max_x - m_min_x; }
	void width( const double &new_width ) { m_max_x = m_min_x + new_width; }
	double height() const { return m_max_y - m_min_y; }
	void height( const double &new_height ) { m_max_y = m_min_y + new_height; }
		// Dimensions.

	void include( const point &pt );
		// Add pt to the area covered by the rectangle.

	void offset( const point &pt );
		// Offset the position of the rectangle by pt.

	void inflate_to_aspect_ratio( const double &aspect_ratio );
		// Increase the size of the rectangle to match the given aspect ratio.

private:

	double m_min_x;
	double m_min_y;
	double m_max_x;
	double m_max_y;
		// Coordinates of the rectangle.
};


}; // namespace eph


#endif // EPHEMERA_GEOMETRY_H
