// ephemera/global.h
// Configuration and common include files for Ephemera.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#ifndef EPHEMERA_GLOBAL_H
#define EPHEMERA_GLOBAL_H


// Tell MSVC to be quiet about useless warnings.
#ifdef _MSC_VER
	#pragma warning( disable : 4996 )
	#define _CRT_SECURE_NO_WARNINGS
#endif // _MSC_VER


// Common includes.
#ifndef RC_INVOKED
	#include <cmath>
	#include <cstdlib>
	#include <ctime>
	#include <random>
	#include <string>
	#include <vector>
#endif // !RC_INVOKED
#include <Windows.h>
#ifdef RC_INVOKED
	#include <WinRes.h>
#endif // RC_INVOKED
#ifndef RC_INVOKED
	#include <TChar.h>
	#include "debug.h"
#endif // !RC_INVOKED


// Ensure we're using templated min and max, not the macros.
#undef min
#undef max


// Provide PI if not already defined.
#ifndef M_PI
	#define M_PI	3.14159265358979323846
#endif // M_PI


// Number of items in an array.
#ifndef numberof
	#define numberof( A ) (sizeof( A )/sizeof( A[0] ))
#endif // numberof


// Convert a symbolic value to a string.
#define EPH_STRINGIFY2( X )		#X
#define EPH_STRINGIFY( X )		EPH_STRINGIFY2( X )


#ifndef RC_INVOKED

namespace eph
{


// Epsilon (minimum significant value).
#define EPH_EPSILON		1e-9
inline bool epsilon_z( double v ) { return fabs( v ) < EPH_EPSILON; }
inline bool epsilon_nz( double v ) { return fabs( v ) >= EPH_EPSILON; }
inline bool epsilon_pos( double v ) { return v >= EPH_EPSILON; }


// Signed square root: sign( x ) * sqrt( abs( x ) ).
inline double ssqrt( const double &x ) { return (x >= 0.0) ? sqrt( x ) : -sqrt( -x ); }


// Clamp a value to [minValue, maxValue].
template<typename T>
T
clamp( T value, T minValue, T maxValue )
{
	if ( value < minValue )
	{
		return minValue;
	}
	else if ( value > maxValue )
	{
		return maxValue;
	}
	return value;
}


// Base class for objects that shouldn't be copied.
class uncopyable
{
public:
	
	uncopyable() {}
		// Allow regular (non-copy) construction.
	
private:
	
	uncopyable( const uncopyable & );
	uncopyable &operator=( const uncopyable & );
		// Disable copy construction and assignment.
};


}; // namespace eph

#endif // !RC_INVOKED


#endif // EPHEMERA_GLOBAL_H
