// ephemera/matrix3.h
// Simple matrix class used for transformations.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#ifndef EPHEMERA_MATRIX3_H
#define EPHEMERA_MATRIX3_H


namespace eph
{


// Simple matrix class used for transformations.
class matrix3
{
public:

	matrix3() {}
		// Construction.

	void set( size_t col, size_t row, const double &value ) { assert( col < 3 ); assert( row < 3 ); m_matrix[col][row] = value; }
	const double &get( size_t col, size_t row ) const { assert( col < 3 ); assert( row < 3 ); return m_matrix[col][row]; }
		// Access to elements in the matrix.

	void identity();
	void scale( const double &sx, const double &sy );
	void rotate( const double &radians );
	void shear( const double &x_in_y, const double &y_in_x );
		// Set the matrix to the identity matrix or a standard transformation.

	matrix3 operator*( const matrix3 &m ) const;
		// Multiply two matrices.

private:

	double m_matrix[3][3];
		// Matrix contents arranged as [column][row].
};


}; // namespace eph


#endif // EPHEMERA_MATRIX3_H
