// ephemera/random.cpp
// Wrapper for pseudo-random number generation.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#include "global.h"

#include "random.h"


namespace eph
{


// Random numbers.
std::mt19937 random::m_random( (unsigned long) time( 0 ) );


//
// Get a random number in the given range.
//

int
random::get( int min_value, int max_value )
{
	assert( min_value <= max_value );
	std::uniform_int_distribution<> dist( min_value, max_value );
	return dist( m_random );
}


//
// Get a random number in the given range.
//

int
random::get( unsigned int min_value, unsigned int max_value )
{
	assert( min_value <= max_value );
	std::uniform_int_distribution<> dist( min_value, max_value );
	return dist( m_random );
}


//
// Get a random number in the given range.
//

double
random::get( double min_value, double max_value )
{
	assert( min_value <= max_value );
	std::uniform_real_distribution<> dist( min_value, max_value );
	return dist( m_random );
}


}; // namespace eph
