// ephemera/renderer.h
// Management of fractal drawing.
// Copyright (c) 2015 Ben Michell <http://www.muryan.com/>.

// This file is part of Ephemera.
//
// Ephemera is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Ephemera is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Ephemera.  If not, see <http://www.gnu.org/licenses/>.

#ifndef EPHEMERA_RENDERER_H
#define EPHEMERA_RENDERER_H


#include "ifs_fractal.h"


namespace eph
{


// Colour systems for applying generated points to the display.
enum colour_system
{
	cs_overwrite,	// Set each pixel.
	cs_merge,		// Merge new pixels with the existing colour.
	cs_intensity	// Gradually increase the content of each pixel.
};


//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


// Management of fractal drawing.
class renderer
{
public:

	renderer( unsigned char *bitmap, int width, int height, int bytes_per_row );
		// Construction.

	int width() const { return m_width; }
	int height() const { return m_height; }
		// Dimensions of the bitmap.

	double aspect_ratio() const { return double( m_width ) / double( m_height ); }
		// Aspect ratio of the bitmap.

	void fractal( const ifs_fractal &ifs, const rectangle &rect, colour_system drawing_method );
		// Set the fractal to draw.

	void clear();
		// Clear the bitmap to black.

	void draw( long point_count );
		// Draw some points into the bitmap.

private:

	static const size_t m_point_buffer_size;
		// Number of points to provide in the point generation buffer.

	unsigned char *m_bitmap;
	int m_width;
	int m_height;
	int m_bytes_per_row;
		// Target bitmap for drawing.

	ifs_fractal m_fractal;
		// Fractal being drawn.

	double m_offset_x;
	double m_offset_y;
	double m_scale_x;
	double m_scale_y;
		// Transformation between fractal space and image space.

	colour_system m_drawing_method;
		// Method for drawing the fractal.

	std::vector<ifs_point> m_point_buffer;
		// Buffer used for point generation.
};


}; // namespace eph


#endif // EPHEMERA_RENDERER_H
